(* Modular math ops
   Copyright (C) 2002 Shawn Wagner <raevnos@pennmush.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.
	 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
	 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)

(** Type-generic math, constants, useful functions, etc. *)

(** {1 Modules that allows functors that do generic math without needing a specific type} *)

(** This describes the type of math. *)

(** *)
type integer
type real

(** This describes the type-generic operations that can be performed *)
module type Ops = sig
  type t
  type num_type
  val add: t -> t -> t
  val sub: t -> t -> t
  val succ: t -> t
  val pred: t -> t
  val mul: t -> t -> t
  val div: t -> t -> t
  val rem: t -> t -> t
  val abs: t -> t
  val zero: t
  val one: t
  val min: t
  val max: t
  val print: out_channel -> t -> unit
  val to_float: t -> float
  val of_float: float -> t
end

(** {2 Specializations for all the built-in integer and real types} *)
module IntOps : (Ops with type num_type = integer and type t = int)
module Int32Ops : (Ops with type num_type = integer and type t = int32)
module Int64Ops : (Ops with type num_type = integer and type t = int64)
module NativeOps : (Ops with type num_type = integer and type t = nativeint)
module FloatOps : (Ops with type num_type = real and type t = float)

(** {1 Useful functions} *)

(** Returns [n+1.0] *)
val succ_float: float -> float

(**  Returns [n-1.0] *)
val pred_float: float -> float

(** Like [compare], but takes an extra epsilon value to use in figuring
out if the floats are 'close enough' to be considered equal. See The
Art of Computer Programming, 4.2.2. As an example,
[fcmp ~epsilon:0.00001 5.000005 5.000006] returns 0, meaning
5.000005 ~~ 5.000006. *)
val fcmp: epsilon:float -> float -> float -> int

(** {2 Classification} *)

val isfinite: float -> bool
(**
  @return True if x is a finite value
*)

val isinf: float -> bool
(**
  @return True if x is infinite
*)

val isnan: float -> bool
(**
  @return True if x is nan 
*)

val isnormal: float -> bool
(**
  @return True if x is a normal number
*)

val iszero: float -> bool
(**
  @return True if x is 0.0 or -0.0
*)

(** {2 C99 functions} *)

(** If these functions aren't provided by a system's libc, they raise Failure. *)

(** The C99 function fma() *)
external fma: float -> float -> float -> float = "stew_fma" "fma" "float"

(** The C99 function fdim(). *)
external fdim: float -> float -> float = "stew_fdim" "fdim" "float"
(**
 @return The positive difference between its arguments.
*)

(** The C99 function nextafter(). *)
external nextafter: float -> float -> float = "stew_nextafter" "nextafter" "float"
(**
 @return The next representable value after x in the direction of y.
*)

(** The C99 function remainder() *)
external remainder: float -> float -> float = "stew_remainder" "remainder" "float"
(**
  @return x REM y. See IEC 60559.
*)

(** The C99 function trunc() *)
external trunc: float -> float = "stew_trunc" "trunc" "float"
(**
  @return The integer value nearest to but no larger in magnitude than x.
*)

(** The C99 function round() *)
external round: float -> float = "stew_round" "round" "float"
(**
  @return X rounded to the nearest integer value, rounding halfway cases away from zero.
*)


(** The C99 function nearbyint() *)
external nearbyint: float -> float = "stew_nearbyint" "nearbyint" "float"
(**
  @return x, rounded to the nearest integer value, using the current rounding direction.
*)

(** The C99 function tgamma() *)
external tgamma: float -> float = "stew_tgamma" "tgamma" "float"
(**
  @return The gamma function of x.
*)

(** The C99 function lgamma() *)
external lgamma: float -> float = "stew_lgamma" "lgamma" "float"
(**
  @return The natural logarithm of the absolute value of gamma of x.
*)

(** The C99 function erfc() *)
external erfc: float -> float = "stew_erfc" "erfc" "float"
(** 
  @return The complementry error function of x.
*)

(** The C99 function erf() *)
external erf: float -> float = "stew_erf" "erf" "float"
(**
  @return The error function of x.
*)

(** The C99 function hypot() *)
external hypot: float -> float -> float = "stew_hypot" "hypot" "float"
(**
  @return The square root of the sum of the squares of x and y.
*)

(** The C99 function cbrt() *)
external cbrt: float -> float = "stew_cbrt" "cbrt" "float"
(**
  @return the cube root of its argument
*)

(** The C99 function scalbn() *)
external scalbn: float -> int -> float = "stew_scalbn" 
(**
  @return x * (FLT_RADIX ^ n)
*)

(** The C99 function logb() *)
external logb: float -> float = "stew_logb" "logb" "float"
(**
  @return the exponent of x, as a signed integer value in float format.
*)

(** The C99 function log2() *)
external log2: float -> float = "stew_log2" "log2" "float"
(**
  @return The log base 2 of its argument.
*)

(** The C99 function log1p() *)
external log1p: float -> float = "stew_log1p" "log1p" "float"
(**
  @return The base-e logarithm of 1 plus x.
*)

(** The C99 function ilogb() *)
external ilogb: float -> int = "stew_ilogb" 
(**
  @return The exponent of x.
*)

(** The C99 function expm1() *)
external expm1: float -> float = "stew_expm1" "expm1" "float"
(**
  @return The base-e exponential of x, minus 1.
*)

(** The C99 function exp2() *)
external exp2: float -> float = "stew_exp2" "exp2" "float"
(**
  @return The base-2 exponential of x.
*)

(** The C99 function atanh() *)
external atanh: float -> float = "stew_atanh" "atanh" "float"
(**
  @return The hyperbolic arc cotangent of x.
*)

(** The C99 function asinh() *)
external asinh: float -> float = "stew_asinh" "asinh" "float"
(**
  @return The hyperbolic arc sine of x.
*)

(** The C99 function acosh() *)
external acosh: float -> float = "stew_acosh" "acosh" "float"
(**
  @return The hyperbolic arc cosine of x.
*)

(** {1 Useful constants} *)

val e: float
(** Euler? ... Euler? ... Euler? *)

val log2e: float
(** [Math.log2 Math.e] *)

val log10e: float
(** [log10 Math.e] *)

val ln2: float
(** [log 2] *)

val ln10: float
(** [log 10] *)

val pi: float
(** Cherry *)

val pi2: float
(** [Math.pi /. 2.] *)

val pi4: float
(** [Math.pi /. 4.] *)

val invpi: float
(** [1. /. Math.pi] *)

val invpi2: float
(** [2. /./ Math.pi] *)

val sqrtpi2: float
(** [2. *. sqrt Math.pi] *)

val sqrt2: float
(** [sqrt 2.] *)

val invsqrt2: float
(** [1. /. sqrt 2.] *)
