(* Mersenne Twister PRNG routines for ocaml
   Copyright (C) 2002 Shawn Wagner <raevnos@pennmush.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.
	 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
	 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)

(** Mersenne Twister PRNG *)

(** Ocaml version of the Mersenne Twister random number generator. 

The interface of this pretty much follows the C reference version. It also produces the same output for a given seed as the reference version, of course.

@see <http://www.math.keio.ac.jp/matumoto/emt.html> for more information.

*)

(** {1 The seed functions.} *)

(** Only one call to one of these is needed. This should be enough variety. *)

(** Seed from an int32 value. *)
val init32: int32 -> unit

(** Seed from an int value. *)
val init: int -> unit

(** Seed from an int32 array of any length. *)
val init_array32: int32 array -> unit

(** Seed from an int array of any length. *)
val init_array: int array -> unit

(** Seed from an int32 bigarray of any length. *)
val init_bigarray32: (int32, Bigarray.int32_elt, 'a) Bigarray.Array1.t -> unit

(** Seed from an int32 array generated by calling Array.map on f and the array. *)
val init_genarray: ('a -> int32) -> 'a array -> unit

(** Seed from raw data in a file. The files used should hold at least 700 ints *)
val init_from_file : string -> unit
val init_from_channel: in_channel -> unit
val init_from_descr: Unix.file_descr -> unit

(** Seed from /dev/urandom *)
val urandom_found: unit -> bool
val urandom_init: unit -> unit

(** Seed from /dev/urandom, or a default seed based on time and pid if
   /dev/urandom isn't available  *)
val self_init: unit -> unit

(** {1 Functions returning random numbers} *)

(** All of these but the int64 ones produce the same output
as the reference C code. *)


(** Unsigned int32. Range: 0 <= x <= 0xffffffff *)
val uint32: unit -> int32

(** Signed int32. Range: 0 <= x <= 0x7fffffff *)
val int32: unit -> int32

(* Unsigned int64. Range: 0 <= x <= 0xffffffffffffffff *)
val uint64: unit -> int64

(* Signed int64. Range: 0 <= x <= 0x7fffffffffffffff *)
val int64: unit -> int64

(** Unsigned nativeint. *)
val unativeint: unit -> nativeint

(** Signed nativeint *)
val nativeint: unit -> nativeint

(** Unsigned int. Range: 0 <= x <= 0x7fffffff or 0 <= x <= 0x7fffffffffffffff *)
val uint: unit -> int

(** Signed int. Range: 0 <= x <= 0x3fffffff or 0 <= x <= 0x3fffffffffffffff *)
val int: unit -> int

(** Range: 0 <= x <= 1 *)
val real1: unit -> float

(** Range: 0 <= x < 1 *)
val real2: unit -> float

(** Range: 0 < x < 1 *)
val real3: unit -> float

(** Range: 0 <= x < 1 with 53-bit resolution *)
val res53: unit -> float

(** {1 Sources for use with the [Rand] distributions}  *)
module IntSource : (Rand.RNGSource with type t = int)
module Int32Source : (Rand.RNGSource with type t = int32)
module FloatSource : (Rand.RNGSource with type t = float)
